#define IS icon_state					//	If you see IS in the program, it is actually icon_state (think define as a shortcut: #define __SHORTCUT__ __VALUE__)
atom/icon='Demo.dmi'					//	All default atom icon file is from Demo.dmi
turf/IS="grass"							//	IS is defined as icon_state. Nice shortcut, neh?

mob
	Entered(Item/A)						//	If an item or anything else sucessfully enters the mob's contents
		..()							//	Checks to see if there's anything else to be called (in case).
		if(istype(A))
			var/Item/X=locate(A.type) in usr.contents-A	//Checks if there's an item with the same type already.
			if(X)						//	If there's an item that exists with the same path
				A.amount+=X.amount		//	Adds the amount of the item found
				del X					//	Deletes that item found.
		Inventory()						//	Calls the Inventory() prcoedure.

	Exited()
		..()
		Inventory()

	proc/Inventory()
		var/i=1												//	Current position (#) of contents
		winset(src,"grid",{"cells="2x[contents.len+1]""})	//	Sets column = 2, row = contents.len+1
		src<<output("<b>Items</b>","grid:1,1")				//	Displays "Items" in the current cell, which is column 1, row 1
		src<<output("<b>Amount</b>","grid:2,1")				//	Displays "Amount" in column 2, row 2
		for(var/Item/I in contents)							//	Searches through the contents list
			src<<output(I,"grid:1,[++i]")					//	Outputs to the cell: column=1, row = ++i (++i means (i+1) returned immediately. i++ means (i+1) but first returns the value of i).; this Sets the item (I) in to the current cell
			src<<output(I.amount,"grid:2,[i]")				//	Displays the amount the item (I) has in to the cell: column=2, row = i (i is already now i+1 from the previous output())

	verb/Clear_Text()										//	To clear the chat part.
		src<<output(null,"output")

client/New()							//	When a client logs in to the game
	winset(src,"map","icon-size=32")	//	Sets the map setting to 32x32 ico (no stretching)
	..()								//	Allows the default client/New() to happen, which does not result in the client being disconnected due to no /mob

//	As an important note, NEVER EVER EVER call usr in movement procedures. Usr is generally acceptable in just verbs, Stat() and Click()/DblClick(). Read the definition of 'usr' to see why.
atom/movable							//	In the hierarchy, atom/movable is the parent to the two commonly used mobile datums known as /mob and /obj ... so anything that effects this parent effects the children as well.
	Bump(atom/A)						//	Redefining (overwriting) Bump(). For those that don't know, Bump() is called on the object INTO something else.
		A.Bumped(src)					//	Making it so the object that was bumped calls Bumped() with the Bump() object as the argument
		..()							//	As we overwrote Bump(), we want to continue whatever there was before. To do this, we call the parent procedure ..()

atom/proc/Bumped(atom/A) return 1		//	And here is where we define the default for Bumped().. Note it is under /atom not /atom/movable. /atom children are /area, /turf and /movable [hence atom/movable]. The bumped() object does not necessarily have to be just an /obj or /mob right?

Item									//	A datum named Item for items (obviously).
	parent_type = /obj					//	/Item's parent (which can be checked via atom.parent_type) is now set to /obj, meaning that /Item contains all traits of /obj which contains it from /atom/movable which contains it from /atom (modified along the way of course).
	density = 1							//	Makes the item dense (unable to walk through)
	var/amount = 1						//	Stacking system!
	Bumped(mob/M)						//	If something Bump()s into it, this is called.
		if(!ismob(M)) return 0			//	If M (the argument sent) is not a mob, the rest is stopped from happening
		if(!M.client) return 0			//	If M is not a client (human player), the rest of the procedure does not happen.
		Pickup_proc(M)					//	Calls the procedure Pickup_proc.

	verb/Pickup()						//	All /Item have this verb.
		set src in oview(1)				//	Makes the verb available if the user if within 1 tile from the source.
		set category = "Items"			//	The verb's category is set as Items
		Pickup_proc(usr)				//	You know what this does.

	verb/Drop()							//	All /Item have this verb as well.
		set src in usr					//	Verb is available if the item is in the contents.
		set category = "Items"			//	The verb's category is set as Items
		var/amt=max(0,min(99,input("How many [name] do you wish to drop? Maximum amount: [min(99,amount)]","Item drop",1) as num|null))
		if(!(src in usr)||!amt) return 0		//	Safety checking if the item is actually in the usr's contents.
		var/turf/XX = get_step(usr,turn(usr.dir,180))
		if(!XX)XX=usr.loc
		if(amt>=amount).=Move(XX)	//	Tries to moves the item to one step behind the user.
		else
			var/Item/X = type
			X = new X(XX)
			X.amount = amt
			amount-=amt
			.=1
			usr.Inventory()
		usr << "[(.)?"\green":"\red"] You[(.)?"":" did not"] drop [amt] [src.name]\s."	//	Checks if the item was dropped sucessfully. (x)?y:z is a mini-if() statement: if(x){ y } else { z }

	proc/Pickup_proc(mob/M)				//	Reason why I defined this instead of just having the verb is so I can pick up /Item's by bumping into them. Sure I could call the verb through Bumped() but than the usr would be someone else.
		if(!(M in oview(1,src))) return	//	Checks if M is within 1 tile. If it isn't, the procedure stops.
		.=Move(M)	//	Tries to moves the item to one step behind the user.
		M << "[(.)?"\green":"\red"] You[(.)?"":" did not"] pick up [src.name]."	//	Checks if the item was picked up sucessfully. (x)?y:z is a mini-if() statement: if(x){ y } else { z }

	Apple/IS="apple"					//	Our first item: /Item/Apple
	Banana/IS="banana"
	Orange/IS="orange"

	New()
		..()
		mouse_drag_pointer = icon(src.icon, src.icon_state)  //  Makes the drag icon the item itself - useless really since MosueDrop() is not used